import argparse
from dotenv import load_dotenv

from jemma.tools import parse_cli_arguments
from jemma.requirements.feature import Feature
from jemma.team.business_owner import BusinessOwner
from jemma.team.engineer import Engineer
from jemma.team.tester import Tester
from jemma.team.project_manager import ProjectManager
import jemma.work.flow as flow

import jemma.thinker

def main():

    ## ----------------------------- setup
    load_dotenv()
    args = parse_cli_arguments()
    brain = thinker.make_brain(args)

    ## ----------------------------- create a feature
    # read requirements from the file
    requirements = ""
    if args.requirements:
        with open(args.requirements, 'r') as file:
            requirements = file.read()

    feature = Feature(requirements)

    ## ----------------------------- create a team
    business_owner = BusinessOwner("an experienced business owner with attention to detail, "
                                   "focused on building requirements for engineers to build software products")

    engineer = Engineer("an experienced software engineer "
                        "with a focus on full stack development")

    tester = Tester("a professional software tester with a deep expertise is "
                    "understanding business requirements and how they translate into software features")

    project_manager = ProjectManager(feature)

    ## ----------------------------- rock & roll

    if args.user_stories:
        flow.create_user_stories (brain,
                                  project_manager,
                                  business_owner,
                                  engineer)
    if args.build_prototype:
        flow.build_prototype(brain,
                             project_manager,
                             business_owner,
                             engineer,
                             args.prompt)

    if args.build_user_stories:
        flow.build_user_stories(brain,
                                project_manager,
                                business_owner,
                                engineer)
    if args.test_prototype:
        flow.test_prototype(brain,
                            project_manager,
                            tester)

if __name__ == '__main__':
    main()



import json
from jemma.requirements.feature import UserStory
from jemma.tools import read_file, say, color
import jemma.prompt.business.owner as prompt

class BusinessOwner:
    def __init__(self, role, title="Business Owner"):
        self.role = role
        self.title = title

    def split_requirements_to_features(self, thinker, requirements, features="", evaluation=""):
        features = thinker.think(prompt.split_requirements_to_features(self.role,
                                                                       requirements,
                                                                       features,
                                                                       evaluation),
                                 self.title)
        splits = [line for line in features.strip().split('\n') if line.strip()]
        return splits

    def evaluate_features(self, thinker, features, requirements):
        features_json = json.dumps(features)
        evaluation = thinker.think(prompt.evaluate_features(self.role, features_json, requirements), self.title)
        return evaluation

    def refine_feature(self, thinker, requirements, feature):
        refined_feature = thinker.think(prompt.refine_feature(self.role, requirements, feature), self.title)
        return refined_feature

    def split_and_refine(self,
                         thinker,
                         requirements,
                         max_refinement_attempts=1):

        # slit requirements into features
        features = self.split_requirements_to_features(thinker, requirements)

        # evaluate and split features until approved or max attempts reached
        refinement_attempts = 0
        while refinement_attempts < max_refinement_attempts:
            evaluation = self.evaluate_features(thinker, features, requirements)
            if "APPROVED AND READY FOR REFINEMENT" in evaluation:
                break
            features = self.split_requirements_to_features(thinker, requirements, features, evaluation)
            refinement_attempts += 1

        if refinement_attempts == max_refinement_attempts:
            say(self.title, "end of day for me (\"max refinement attempts\" is reached)")
        else:
            say(self.title, "we did well: requirements are split into features and approved for refinement")

        # refine each feature individually
        refined_features = []
        for feature in features:
            refined_feature = self.refine_feature(thinker, requirements, feature)
            feature_map = {"title": feature,
                           "acceptance_criteria": refined_feature}
            refined_features.append(feature_map)

        return refined_features

    def clarify_user_story(self,
                           thinker,
                           requirements,
                           user_story,
                           clarification_request):

            # use the thinker to clarify the user story
            clarified = thinker.think(prompt.clarify_user_story(self.role,
                                                                requirements,
                                                                user_story,
                                                                clarification_request),
                                           self.title)

            user_story.update_acceptance_criteria(clarified)

            return user_story

    def combine_user_stories(self, thinker, requirements, user_stories):
        say(self.title, "combining user stories...", message_color=color.DARKCYAN)
        return thinker.think(prompt.combine_user_stories(self.role, requirements, user_stories),
                             self.title)


    def idea_to_prompt(self, thinker, idea):
        say(self.title, "📚 creating detailed requirements ...🖋️", message_color=color.DARKCYAN)
        return thinker.think(prompt.idea_to_prompt(idea),
                             self.title,
                             mute=True)



def create_css_file(requirements):
    return f"""
You are a CSS expert with deep knowledge of Twitter Bootstrap CSS framework.

Your task is to generate a complete and working CSS file for a web prototype based on a twitter's "bootstrap.css" framework and the following requirements:

# REQUIREMENTS
{requirements}

follow these guidelines to create the CSS file:

# GUIDELINES
- Implement a color scheme with pastel colors that creates a pleasing atmosphere.
  - USE colors on a brighter side effectively to create visual hierarchy, highlight important elements, and convey the desired mood.
- Use appropriate margins, paddings, and spacing to create a balanced and visually pleasing arrangement.
  - (!) make sure DataTables are styled and flexible width so they don't overflow
- Choose a clean and readable font family for headings and body text
  - make font small enough to fit the content
- Make sure top and side margins and paddings make user focus on the main content
- Use Bootstrap's grid system to create a responsive layout
  - align menu items in a single row whether it is vertical or horizontal

Please generate a complete CSS file that fulfills these requirements and provides a beautiful, user-friendly, and responsive design for the web prototype. The generated CSS file should be ready to be integrated with the corresponding HTML and JavaScript files.

do NOT surround the file with markdown backticks: ```css ... ```
start the response with /* Global Styles */ and follow with CSS source according the guidelines.

# IMPORTANT
Provide the CSS file content ONLY, without any additional text or explanations. The CSS file should be well-structured, organized, and easy to understand for developers working on the web prototype.

(!) make sure ALL the requirements and actions are addressed in the CSS file.
"""


def create_javascript_file(requirements, css):
    return f"""
You are a JavaScript expert who specializes in jQuery with a focus on Twitter Bootstrap and DataTables libraries.

Given requirements and a CSS file (based on "bootstrap.css") your task is to generate a JavaScript file that utilizes the jQuery library to create an interactive and dynamic web prototype based on these requirements that relies on that CSS file.

for tables, use a DataTables jQuery plugin to enhance the functionality and appearance of the tables.

This JavaScript file should be self-contained as this would be the only file that would be used to run the web prototype.

# REQUIREMENTS
{requirements}

# CSS FILE
{css}

follow these guidelines to create the JavaScript file:

# GUIDELINES
- Make sure layout pieces do not overlap or are not hidden by other elements.
- Make sure tables are used to store grid and tabular data
- Ensure that the generated JavaScript file seamlessly integrates with the provided CSS file.
- Instead of calling backend APIs populate test data or MINIMUM 7 entries in the JavaScript file to simulate the dynamic behavior of the web prototype.
  - doublechek that the data is visible to the user
  - doublecheck that the data is POPULATED
- Make sure edits happen in either inline within the table rows or in a modal window.
- do not forget to close all the code forms in parentheses, brackets, etc. $(function() {{ foo = [..] }});
- don't use todo action comments such as "// Add more test data here" or "// add code for to do X": instead IMPLEMENT the data / action

do NOT surround the file with markdown backticks: ```javascript ... ```
- start your response with a comment "// jQuery implementation"
- followed by the source code according to guidelines.
- ending with a comment "// prototype implementation"

IMPORTANT: Provide the JavaScript file content ONLY, without any additional text or explanations. The JavaScript file should be well-structured, organized, and easy to understand for developers working on the web prototype.

# IMPORTANT
(!) make sure ALL the requirements and actions are addressed in the JavaScript file.
"""

def create_html_file(requirements, css, js):
    return f"""
Code Generator Agent,

Given requirements, a CSS file (based on "bootstrap.css"), and a JavaScript file, your task is to generate a complete and semantic HTML file that serves as the foundation for the web prototype based on these requirements.
The HTML file should import these in the following order:

- remote "bootrap.css"
- remote dataTables css (for bootstrap 4, "dataTables.bootstrap4.css")
- local, previously generated, CSS: app.css
- remote jQuery library
- remote "popper.js"
- remote "bootrap.js"
- remote dataTables js
- remote dataTables js (for bootstrap 4, "dataTables.bootstrap4.js")
- local, previously generated, JavaScript: app.js

example:

  <link rel="stylesheet" type="text/css" href="https://cdn.jsdelivr.net/npm/bootstrap@4.4.1/dist/css/bootstrap.min.css">
  <link href="https://cdn.datatables.net/2.0.3/css/dataTables.bootstrap4.css" rel="stylesheet">
  <link rel="stylesheet" type="text/css" href="app.css">
  <script src="https://code.jquery.com/jquery-3.6.0.min.js"></script>
  <script src="https://cdn.jsdelivr.net/npm/popper.js@1.16.0/dist/umd/popper.min.js"></script>
  <script src="https://cdn.jsdelivr.net/npm/bootstrap@4.4.1/dist/js/bootstrap.min.js"></script>
  <script src="https://cdn.datatables.net/2.0.3/js/dataTables.js"></script>
  <script src="https://cdn.datatables.net/2.0.3/js/dataTables.bootstrap4.js"></script>
  <script src="app.js"></script>

and adhere to the following requirements to create the best layout and user experience possible.

# REQUIREMENTS
{requirements}

# CSS FILE
{css}

# JAVASCRIPT FILE
{js}

follow these guidelines to create the HTML file:

# GUIDELINES

- Link the previously generated CSS file in the head section using the appropriate `<link>` tag.
- Place the previously generated JavaScript file before the closing `</body>` tag using the `<script>` tag.
- Ensure that the file paths for the CSS and JavaScript files are correct and relative to the HTML file.

- do NOT create elements that are not required by the user story.

- rely on the BOOTSTRAP GRID SYSTEM to create a responsive layout.
  - (!) make sure the grid pieces, including offsets, in a single row do not exceed 12 columns.
  - (!) don't waste space, if an box (such as a "side menu") needs to be "col-1" or "col-2" make it so
- (!) make sure layout pieces do not overlap or are not hidden by other elements.
- make sure tables are used to store grid and tabular data

Please generate a complete and semantic HTML file that fulfills these requirements and creates the best layout and user experience possible. The generated HTML file should seamlessly integrate with the previously generated CSS and JavaScript files to form a cohesive web prototype.

do NOT surround the file with markdown backticks: ```html ... ```
start the response with <!DOCTYPE html> and follow the guidelines.

IMPORTANT: Provide the HTML file content ONLY, without any additional text or explanations: no text headers, no footers. The HTML file should be well-structured, organized, and easy to understand for developers working on the web prototype.

# IMPORTANT
(!) make sure ALL the requirements and actions are addressed in the HTML file.
"""

def implement_user_story(approach,
                         requirements,
                         user_story,
                         existing_code = ""):
    return f"""You are an AI agent acting as an experienced software engineer. Your task is to implement a user story based on the provided approach, project requirements and existing code.

(!) provide NO EXPLAINATION, only the source code.

Follow these steps:
1. Review the project requirements for context and constraints.
2. Analyze the user story in Gherkin format and its acceptance criteria.
3. Implement the user story using the specified approach, ensuring it meets the acceptance criteria and project requirements.
4. Provide a complete, working implementation of the user story, including any necessary code, database schemas, API endpoints, or UI components.
5. Include comments in your code to explain key decisions, assumptions, or complex logic.
6. If any part of the implementation is unclear or requires further discussion, highlight it with a comment.
7. If existing code is provided, integrate your implementation with it, ensuring compatibility and consistency.

Approach:
{approach}

Project Requirements:
{requirements}

User Story:
{user_story}

Existing Code:
{existing_code}

extract ONLY source code from the existing code to reference it in your implementation.
"""

def default_implementation_approach():
    return """create a simple, but WORKING prototype.
use no libraries, dependencies or frameworks.
for UI use HTML, CSS and JavaScript.
for backend use Python
"""

def ui_prototype_implementation_approach():
    return """
Create a functional UI prototype that demonstrates the core functionality of the user story.

Guidelines:
- Use only HTML, CSS, and JavaScript in a single file to make it immediately runnable from the browser.
- Structure the HTML semantically, using appropriate tags for headings, sections, navigation, etc.
- Use inline CSS styles within the <style> tag in the <head> section of the HTML file.
- Implement the necessary interactivity using JavaScript within the <script> tag at the end of the <body> section.
- Use meaningful and diverse stub data that closely resembles real-life scenarios.
- If the user story requires data persistence, use the browser's local storage to store and retrieve data.
- Organize the code into separate sections for HTML, CSS, and JavaScript using comments for clarity.
- Keep the code clean, readable, and properly indented.
- Center the main content on the page, while positioning specific components (e.g., side menu) as required.
- Choose a simple color palette with 2-3 primary colors and appropriate contrast.
- Use a smooth, readable font for body text and a complementary font for headings.
- Ensure adequate whitespace and padding around elements to improve readability.
- Handle user input and events (e.g., clicks, form submissions) appropriately.
- Update the UI dynamically based on user actions and data changes.
- Validate user input and provide clear feedback and error messages.
- Test the prototype thoroughly across different screen sizes and devices to ensure responsiveness.

Example of the file structure:

<!DOCTYPE html>
<html>
<head>
  <title>UI Prototype</title>
  <style>
    /* CSS styles go here */
  </style>
</head>
<body>
  <!-- HTML content goes here -->

  <script>
    // JavaScript code goes here
  </script>
</body>
</html>

Remember, the goal is to create a self-contained, functional, and visually appealing prototype that showcases the core aspects of the user story. The prototype should be easily understandable by stakeholders and developers alike.

"""

def refactor_css_file(requirements,
                      prototype_css,
                      feedback):
    return f"""
Given the following information:

Original Requirements:
{requirements}

Existing Prototype CSS (using Twitter Bootstrap):
{prototype_css}

User Feedback:
{feedback}

Please refactor the CSS code, taking into account the original requirements and the user's feedback. Consider the following points:

1. Ensure that the refactored CSS aligns with the original requirements and incorporates the user's suggestions where appropriate.
2. Optimize the CSS for better performance, readability, and maintainability.
3. Use best practices and modern CSS techniques where applicable.
4. Keep in mind that the CSS code is using the Twitter Bootstrap library, so make sure to leverage Bootstrap classes and conventions where appropriate.

do NOT surround the file with markdown backticks: ```css ... ```
start the response with /* Global Styles */ and follow with CSS source according the guidelines.

Please provide only the refactored CSS code without any additional explanations or comments. The code will be written into a separate "app.css" file.
    """

def refactor_javascript_file(requirements,
                             css,
                             prototype_js,
                             feedback):
    return f"""
Given the following information:

Original Requirements:
{requirements}

Refactored CSS (using Twitter Bootstrap):
{css}

Existing Prototype JavaScript (using jQuery and DataTables):
{prototype_js}

User Feedback:
{feedback}

Please refactor the JavaScript code, taking into account the original requirements, the refactored CSS, and the user's feedback. Consider the following points:

1. Ensure that the refactored JavaScript aligns with the original requirements and incorporates the user's suggestions where appropriate.
2. Optimize the JavaScript for better performance, readability, and maintainability.
3. Use best practices and modern JavaScript techniques where applicable.
4. Keep in mind that the JavaScript code is using the jQuery library and the DataTables plugin, so make sure to leverage their features and conventions where appropriate.
5. Ensure compatibility with the refactored CSS code, especially considering the use of Twitter Bootstrap classes and conventions.

do NOT surround the file with markdown backticks: ```javascript ... ```
- start your response with a comment "// jQuery implementation"
- followed by the source code according to guidelines.
- ending with a comment "// prototype implementation"

Please provide only the refactored JavaScript code without any additional explanations or comments. The code will be written into a separate "app.js" file.
    """

def refactor_html_file(requirements,
                       css,
                       js,
                       prototype_html,
                       feedback):
    return f"""
Given the following information:

Original Requirements:
{requirements}

Refactored CSS (using Twitter Bootstrap):
{css}

Refactored JavaScript (using jQuery and DataTables):
{js}

Existing Prototype HTML:
{prototype_html}

User Feedback:
{feedback}

Please refactor the HTML code, taking into account the original requirements, the refactored CSS and JavaScript, and the user's feedback. Consider the following points:

1. Ensure that the refactored HTML aligns with the original requirements and incorporates the user's suggestions where appropriate.
2. Optimize the HTML structure for better semantics, accessibility, and SEO.
3. Use appropriate HTML5 elements and attributes where applicable.
4. Ensure compatibility with the refactored CSS code, especially considering the use of Twitter Bootstrap classes and conventions.
5. Integrate the refactored JavaScript code seamlessly, making sure that all necessary elements have the correct IDs, classes, and data attributes required by jQuery and DataTables.
6. Improve the overall user experience and usability of the HTML page based on the user's feedback.

do NOT surround the file with markdown backticks: ```html ... ```
start the response with <!DOCTYPE html> and follow the instructions.

Please provide only the refactored HTML code without any additional explanations, comments. The code will be written into a separate "index.html" file.
    """



def check_whether_clarification_needed(requirements, user_story):
    return f"""
# INSTRUCTIONS
As an experienced software engineer, your task is to carefully review the provided user story and determine if you have enough information to implement it precisely based on the project requirements.

Thoroughly analyze the user story and identify any missing, ambiguous, or unclear details that could impact the implementation. Consider the following aspects:

- all the necessary functionalities and behaviors clearly defined
- specific requirements about system integration
- any edge cases or exceptional scenarios to consider
- any specific performance criteria or constraints to consider
- how should the feature integrate with existing systems or components
- any security considerations or requirements

you job is to see which of the above options apply to the user story
NONE or some could apply, depending on the user story context

If any information is missing or unclear, provide a list of specific questions or points that require further clarification from the business owner. Each clarification request should be clear, concise, and directly related to the implementation of the user story.

Make sure the clarifications and questions are ONLY about the content the user story itself vs. other non relevant requiements of the project.

Only share the final set of questions wit no headers, no footers, no explanations, just the questions.
(!) Only if the user story is already complete and no further clarification is needed respond with "No clarification needed."

# PROJECT REQUIREMENTS
{requirements}

# USER STORY
{user_story}

# ENGINEER'S RESPONSE
"""



from jemma.tools import say, color, open_local_browser
from jemma.requirements.feature import Feature, UserStory

class ProjectManager:
    def __init__(self, feature, project_name="project", role = "Project Manager"):
        self.feature = feature
        self.role = role
        self.project_name = project_name

    def meet_to_create_user_stories(self,
                                    thinker,
                                    business_owner):
        say(self.role, "\nDear Business Owner, in this meeting we'll work on splitting the requirements and creating user stories.",
            who_color = color.CYAN, message_color = color.YELLOW)

        refined = business_owner.split_and_refine(thinker,
                                                  self.feature.requirements)

        for unit in refined:
            user_story = UserStory(title=unit["title"],
                                   acceptance_criteria=unit["acceptance_criteria"])
            self.feature.add_user_story(user_story)

    def meet_to_refine_user_stories(self,
                                    thinker,
                                    engineer,
                                    business_owner,
                                    max_refinement_attempts = 1):

        say(self.role, "\nDear Engineer, meet the Business Owner, welcome to the refinement meeting, let's refine'em user stories!",
            who_color = color.CYAN, message_color = color.YELLOW)

        for title, user_story in self.feature.get_user_stories().items():
            self._refine_user_story(thinker,
                                    engineer,
                                    business_owner,
                                    user_story,
                                    max_refinement_attempts)

    def _refine_user_story(self,
                           thinker,
                           engineer,
                           business_owner,
                           user_story,
                           max_refinement_attempts):

        refinement_attempts = 0
        engineer_response = engineer.review_user_story(thinker,
                                                       user_story,
                                                       self.feature.requirements)

        while "No clarification needed" not in engineer_response and refinement_attempts < max_refinement_attempts:
            say(self.role, "\nDear Business Owner, the Engineer would like you to clarify details about \"" + user_story.title + "\"",
                who_color=color.CYAN, message_color=color.YELLOW)

            clarified_user_story = business_owner.clarify_user_story(thinker,
                                                                     self.feature.requirements,
                                                                     user_story,
                                                                     engineer_response)
            self.feature.update_user_story(user_story.title,
                                           clarified_user_story)

            refinement_attempts += 1
            if refinement_attempts < max_refinement_attempts:
                engineer_response = engineer.review_user_story(thinker,
                                                               self.feature.get_user_story(user_story.title),
                                                               self.feature.requirements)

    def meet_to_build_prototype(self,
                                thinker,
                                engineer,
                                path = "prototype"):

        say(self.role,
            "\nDear Engineer, in this meeting let's put our heads together to build a prototype based on the requirements.",
            who_color = color.CYAN,
            message_color = color.YELLOW)

        prototype = engineer.create_prototype(thinker,
                                              self.feature.requirements)
        engineer.record_prototype(path,
                                  prototype)
        open_local_browser(path)

        return prototype

    def meet_to_refactor(self,
                         thinker,
                         engineer,
                         prototype,
                         feedback,
                         path = "prototype"):

        say(self.role,
            "\nDear Engineer, we have met with the user and received a valuable feedback. sudo make it better! 🛠️",
            who_color = color.CYAN,
            message_color = color.YELLOW)

        refactored = engineer.refactor_prototype(thinker,
                                                 self.feature.requirements,
                                                 prototype,
                                                 feedback)

        engineer.record_prototype(path,
                                  refactored)
        open_local_browser(path)

        return refactored

    def meet_to_test_prototype(self,
                               thinker,
                               tester,
                               path = "prototype"):

        say(self.role,
            "\nDear Tester, in this meeting let's focus on requirements and built code to test a prototype.",
            who_color = color.CYAN,
            message_color = color.YELLOW)

        prototype = tester.read_prototype(path)
        test_results = tester.review_prototype(thinker,
                                               self.feature.requirements,
                                               prototype)

        return test_results

    def meet_to_combine_user_stories(self,
                                     thinker,
                                     business_owner):

        say(self.role, "\nDear Business Owner, in this meeting we'll combine user stories into a single document",
            who_color = color.CYAN, message_color = color.YELLOW)

        combined_user_stories = business_owner.combine_user_stories(thinker,
                                                                    self.feature.requirements,
                                                                    self.feature.get_user_stories())
        self.feature = Feature(combined_user_stories)

    def user_stories_to_requirement(self):
        self.feature = Feature(self.feature.export_user_stories_titles_and_criteria())

    def meet_to_create_requirements(self,
                                    thinker,
                                    business_owner,
                                    idea):
        say(self.role, "\nDear Business Owner, in this meeting we'll work on creating requirements based on the 💡 idea",
            who_color = color.CYAN, message_color = color.YELLOW)

        requirements = business_owner.idea_to_prompt(thinker, idea)
        self.feature = Feature(requirements)



import os, time
from jemma.tools import color, say

import ollama
import replicate
from anthropic import Anthropic
from openai import OpenAI

class Thinker:

    # defining init method for class
    def __init__(self, model_name):
        self.model_name = model_name

    def __str__(self):
        return f"{self.__class__.__name__} 🧠 {self.model_name} ✅"

    def think(prompt, who="user", sleep_time=0):
        raise NotImplementedError("can't think without a implementation. try creating Ollama that is based on me")

class Ollama(Thinker):

    def __init__(self, model_name):
        super().__init__(model_name)

    def think(self, prompt, who="user", action="", mute=False, sleep_time=0):
        thinking = ollama.generate(model=self.model_name,
                                   prompt=prompt,
                                   options={'num_predict': 4096},
                                   stream=True)
        say(who, action)

        response = ""
        for word in thinking:
            response += word['response']
            if not mute:
                print(word['response'], end="")

        return response

class Replicate(Thinker):

    def __init__(self, model_name):
        super().__init__(model_name)

    def think(self, prompt, who="user", action="", mute=False, sleep_time=0):

        say(who, action)

        response = ""
        for event in replicate.stream(
            self.model_name,
            input={
                "prompt": prompt,
                # "max_new_tokens": 32000,
                # "max_length": 32000
            },
        ):
            response = response + str(event)
            if not mute:
                print(str(event), end="")

        # print(color.RED + ">>" + response + "<<" + color.END)

        return response

class Claude(Thinker):

    def __init__(self, model_name):
        super().__init__(model_name)

        ## make sure ANTHROPIC_API_KEY is exported in shell
        ## or it is exported in a .env file
        self.client = client = Anthropic()

    def think(self, prompt, who="user", action="", mute=False, sleep_time=1): # sleep not to exceed the rate limit

        say(who, action)
        time.sleep(sleep_time)

        response = ""
        with self.client.messages.stream(max_tokens=4096,
                                         messages=[{"role": "user",
                                                    "content": prompt}],
                                         model=self.model_name
        ) as stream:
          for text in stream.text_stream:
              if not mute:
                  print(text, end="", flush=True)
              response += text

        # print(color.RED + ">>" + response + "<<" + color.END)

        return response

class ChatGPT(Thinker):

    def __init__(self, model_name):
        super().__init__(model_name)

        ## make sure OPENAI_API_KEY is exported in shell
        ## or it is exported in a .env file
        self.client = client = OpenAI()

    def think(self, prompt, who="user", action="", mute=False, sleep_time=2): # sleep not to exceed the rate limit

        say(who, action)
        time.sleep(sleep_time)

        response = ""
        stream = self.client.chat.completions.create(
            model=self.model_name,
            messages=[{"role": "user",
                       "content": prompt}],
            max_tokens=4096,
            stream=True,
        )
        for chunk in stream:
            thought = chunk.choices[0].delta.content or ""
            if not mute:
                print(thought, end="")
            response += thought

        # print(color.RED + ">>" + response + "<<" + color.END)

        return response

## makes a brain from cli arguments
def make_brain(args, default_models = {'claude': 'claude-3-haiku-20240307',
                                       'openai': 'gpt-3.5-turbo',
                                       'ollama': 'gemma:7b-instruct-v1.1-fp16',
                                       'replicate': 'mistralai/mixtral-8x7b-instruct-v0.1'}):

    model_classes = {
        'claude': Claude,
        'openai': ChatGPT,
        'ollama': Ollama,
        'replicate': Replicate
    }

    # make it with the first model argument that is provided
    for model_name in model_classes.keys():
        model_arg = getattr(args, model_name, None)
        if model_arg is not None:

            model_name_to_use = default_models[model_name] if model_arg is True else model_arg

            brain = model_classes[model_name](model_name_to_use)
            print(color.GRAY_MEDIUM + f"{brain}" + color.END)
            return brain

    # ollama by default
    brain = Ollama(default_models['ollama'])
    print(color.GRAY_MEDIUM + f"{brain}" + color.END)

    return brain

# ---- local models

# nous-hermes2:10.7b-solar-q4_0
# nous-hermes2:10.7b-solar-q6_K
# mistral:7b-instruct-v0.2-fp16
# dolphin-mistral:7b-v2.6-dpo-laser-fp16
# mixtral:8x7b-instruct-v0.1-q4_K_M
# deepseek-coder:6.7b-instruct-fp16
# gemma:7b-instruct-fp16

# ---- claude models

# claude-3-haiku-20240307
# claude-3-sonnet-20240229
# claude-3-opus-20240229

# ---- chatgpt models

# gpt-3.5-turbo-0125   (16K context window)
# gpt-4-32k
# gpt-4-turbo-preview  (128K context window)



import webbrowser
import os, argparse, re

class color:
   PURPLE = '\033[95m'
   CYAN = '\033[96m'
   DARKCYAN = '\033[36m'
   BLUE = '\033[94m'
   GREEN = '\033[92m'
   YELLOW = '\033[93m'
   RED = '\033[91m'
   GRAY_DARK = '\033[38;5;232m'
   GRAY_MEDIUM = '\033[38;5;244m'
   GRAY_LIGHT = '\033[38;5;250m'
   GRAY_VERY_LIGHT = '\033[38;5;254m'
   BOLD = '\033[1m'
   UNDERLINE = '\033[4m'
   END = '\033[0m'

def say(who, message,
        who_color = color.PURPLE,
        message_color = color.CYAN):
   print("\n" + color.BLUE + "> " + color.BOLD + who_color + color.UNDERLINE + who + color.END + ": ", end="")
   print(message_color + message)

def read_file(path):
    with open(path, 'r') as file:
        data = file.read()
    return data

def open_local_browser(dir_path):
   current_dir = os.getcwd()
   try:
      # Get the absolute path of the file
      abs_path = os.path.abspath(dir_path)

      # Check if the file exists
      if os.path.exists(abs_path):
         # Open the file in the default web browser
          webbrowser.open(f"file://{abs_path}/index.html")
          print(f"opened {dir_path} in the web browser")
      else:
         print(f"can't find a file system path to open in the browser: {file_path}")
   finally:
      os.chdir(current_dir)

def parse_cli_arguments():

    parser = argparse.ArgumentParser(description='thoughts in, software out')

    parser.add_argument('--requirements', type=str, help='path to the requirements file')
    parser.add_argument('--prompt', type=str, help='short idea to convert to creation')


    parser.add_argument('--user-stories', action='store_true', help='create and refine user stories')
    parser.add_argument('--build-user-stories', action='store_true', help='build prototype on combined (non refined) user stories')

    parser.add_argument('--build-prototype', action='store_true', help='build prototype')
    parser.add_argument('--test-prototype', action='store_true', help='test prototype')

    parser.add_argument('--claude', nargs='?', const=True, default=None,
                        help='choose a claude model (optional, can be provided without a value)')
    parser.add_argument('--openai', nargs='?', const=True, default=None,
                        help='choose an open ai model (optional, can be provided without a value)')
    parser.add_argument('--ollama', nargs='?', const=True, default=None,
                        help='choose an ollama model (optional, can be provided without a value)')
    parser.add_argument('--replicate', nargs='?', const=True, default=None,
                        help='choose a model from Replicate (optional, can be provided without a value)')

    args = parser.parse_args()

    return args



import readline
from jemma.tools import color

def create_user_stories(brain,
                        project_manager,
                        business_owner,
                        engineer):

    project_manager.meet_to_create_user_stories(brain,
                                                business_owner)

    ## have another business owner to review the stories
    # project_manager.meet_to_review_user_stories(brain,
    #                                            business_owner,
    #                                            stories)

    project_manager.meet_to_refine_user_stories(brain,
                                                engineer,
                                                business_owner)

def build_user_stories(brain,
                       project_manager,
                       business_owner,
                       engineer):
    project_manager.meet_to_create_user_stories(brain,
                                                business_owner)
    # project_manager.meet_to_combine_user_stories(brain,
    #                                              business_owner)
    project_manager.user_stories_to_requirement()
    return project_manager.meet_to_build_prototype(brain,
                                                   engineer)


def ask_for_feedback():

    # make sure the cursor works
    readline.parse_and_bind('set editing-mode emacs')

    while True:
        feedback = input("\n" + color.GREEN + "tell me how to make it better > " + color.END)
        if feedback.strip():
            return feedback
        elif feedback == "":
            return None

def build_prototype(brain,
                    project_manager,
                    business_owner,
                    engineer,
                    prompt = None):

    if prompt:
        project_manager.meet_to_create_requirements(brain,
                                                    business_owner,
                                                    prompt)
    prototype = project_manager.meet_to_build_prototype(brain,
                                                        engineer)

    while True:
        feedback = ask_for_feedback()
        if not feedback:
            break

        prototype = project_manager.meet_to_refactor(brain,
                                                     engineer,
                                                     prototype,
                                                     feedback)

    return prototype

def test_prototype(brain,
                   project_manager,
                   tester):
    return project_manager.meet_to_test_prototype(brain,
                                                  tester)
